#!/bin/bash

#####################################################
# Convert SQLite DB to author based unix variables.
# Arguments:
#   sqlite db
#   entry count
#   config file
# Outputs:
#   Writes author variables to stdout
#####################################################

# print usage info
usage() {
    echo "Usage: ${BASH_SOURCE[0]} -d DB -c etc/config.mk [-a COUNT]"
    exit 1
} ; if [[ "$#" -lt 1 ]]; then usage; fi

# parse options
while getopts ":d:a:c:" opt; do
    case "${opt}" in
        d) sqlite_file="${OPTARG}" ;;
        a) _count="${OPTARG}" ;;
        c) . ${OPTARG} && cflag=true ;;
        :) echo "Error: -${OPTARG} requires an argument." ; usage ;;
        *) usage ;;
    esac
done

# check options
if [[ ! "${cflag}" ]]; then echo "-c option must be specified"; exit 1; fi

# get all author names into the author_list array as unique order, replace "," with newline for parse the multiple authors.
# use awk instead of uniq
readarray -t author_list < <(
    sqlite3 "${sqlite_file}" "SELECT author FROM books ${SQLITE_QUERY}" | sed 's/, /\n/g' | awk '!seen[$0]++'
)

# write authors as sluged url into the author_url array
readarray -t author_url < <(
    printf "%s\n" "${author_list[@]}" | url_slug
)

# read sqlite db into arrays
readarray -t author < <(sqlite3 "${sqlite_file}" "SELECT author FROM books ${SQLITE_QUERY}" | sed "s/'/\&#39;/g")
readarray -t title < <(sqlite3 "${sqlite_file}" "SELECT title FROM books ${SQLITE_QUERY}" | sed "s/'/\&#39;/g")
readarray -t subtitle < <(sqlite3 "${sqlite_file}" "SELECT subtitle FROM books ${SQLITE_QUERY}" | sed "s/'/\&#39;/g")
readarray -t pubdate < <(sqlite3 "${sqlite_file}" "SELECT pubdate FROM books ${SQLITE_QUERY}")
readarray -t directory < <(sqlite3 "${sqlite_file}" "SELECT directory FROM books ${SQLITE_QUERY}")
readarray -t row_id < <(seq 0 ${#author[@]})

# get book title as sluged url into the url array
readarray -t url < <(sqlite3 "${sqlite_file}" "SELECT ${BOOK_SLUG_RULE} FROM books ${SQLITE_QUERY}" | url_slug)

# get author count for loop
total_author="${#author_list[@]}"
array_count="${_count:-$total_author}"

if [[ -z "${_count}" ]]; then
    first_author=$((total_author - array_count))
else
    first_author=$((total_author - array_count - 1))
fi
last_author=$((total_author - 1))


# get first entry after last entry
author_list[$((last_author + 1))]="${author_list[0]}"
author_url[$((last_author + 1))]="${author_url[0]}"


# declare for multidimensional array
echo "declare -A author
author[count]=\"${array_count}\"
author[first]=\"${first_author}\"
author[last]=\"${last_author}\"
"

# functions
function _print_directory() {
	for x in $(seq "${author_first}" "${author_last}"); do
		echo "${directory[${row[$x]}]}"
	done | grep -q "unlisted"
}

# loop for print each author data to env file
for i in $(seq "${first_author}" "${last_author}"); do

            # print grepped author's books row_id into array
            # TODO: find fastest way! Paste, grep and awk make script slow.
            readarray -t row < <(
                    paste <(printf "%s\n" "${row_id[@]}") <(printf "%s\n" "${author[@]}") | \
                    grep "${author_list[$i]}" | \
                    awk '{print $1}'
            )
            # get grepped author's book count
            author_count="${#row[@]}"
            author_first="0"
            author_last="$((author_count - 1))"

    # check items of author if zine or book, TODO: fix that poor way
            case "${directory[${row[0]}]}" in
        zine) text[${row[0]}]="sayıları" ;;
        *) text[${row[0]}]="kitapları" ;;
    esac

    # print author info and its book count
    echo "
author[${i},id]=\"${i}\"
author[${i},name]=\"${author_list[$i]}\"
author[${i},url]=\"${author_url[$i]}\"
author[${i},prev,name]=\"${author_list[$((i - 1))]}\"
author[${i},prev,url]=\"${author_url[$((i - 1))]}\"
author[${i},next,name]=\"${author_list[$((i + 1))]}\"
author[${i},next,url]=\"${author_url[$((i + 1))]}\"
author[${i},count]=\"${author_count}\"
author[${i},first]=\"${author_first}\"
author[${i},last]=\"${author_last}\"
author[${i},text]=\"${text[${row[0]}]}\""


    # find all row_id of author with greping their name from book_env's author and row_id array, and put them into new author array
    for _i in $(seq "${author_first}" "${author_last}"); do
        case "${directory[${row[$_i]}]}" in
            zine)  type[${row[$_i]}]="sayıları" ;;
            *) type[${row[$_i]}]="kitapları" ;;
        esac
echo "author[${i},${_i},author_title]=\"${author[${row[$_i]}]}\"
author[${i},${_i},title]=\"${title[${row[$_i]}]}\"
author[${i},${_i},subtitle]=\"${subtitle[${row[$_i]}]}\"
author[${i},${_i},pubdate]=\"${pubdate[${row[$_i]}]}\"
author[${i},${_i},directory]=\"${directory[${row[$_i]}]}\"
author[${i},${_i},url]=\"${url[${row[$_i]}]}\""
    done

    if _print_directory; then
	echo "author[${i},unlisted]=\"1\""
    else
	echo "author[${i},unlisted]=\"0\""
    fi

done
