#!/bin/bash

#################################################
# Convert SQLite DB to book based unix variables.
# Arguments:
#   sqlite db
#   entry count
#   config file
# Outputs:
#   writes variables to stdout
#################################################

# print usage info
usage() {
    echo "Usage: ${BASH_SOURCE[0]} -d DB -c etc/config.mk [-a COUNT]"
    exit 1
} ; if [[ "$#" -lt 1 ]]; then usage; fi


# parse options
while getopts ":d:a:c:" opt; do
    case "${opt}" in
        d) sqlite_file="${OPTARG}" ;;
        a) _count="${OPTARG}" ;;
        c) . ${OPTARG} && cflag=true ;;
        :) echo "Error: -${OPTARG} requires an argument." ; usage ;;
        *) usage ;;
    esac
done

# check options
if [[ ! "${cflag}" ]]; then echo "-c option must be specified"; exit 1; fi

# read db into bash array
readarray -t uuid < <(sqlite3 "${sqlite_file}" "SELECT uuid FROM books ${SQLITE_QUERY}")
readarray -t title < <(sqlite3 "${sqlite_file}" "SELECT title FROM books ${SQLITE_QUERY}" | sed "s/'/\&#39;/g")
readarray -t author < <(sqlite3 "${sqlite_file}" "SELECT author FROM books ${SQLITE_QUERY}" | sed "s/'/\&#39;/g")
# convert author name to sluged url into url array
readarray -t author_url < <(sqlite3 "${sqlite_file}" "SELECT author FROM books ${SQLITE_QUERY}" | url_slug)
# convert title to sluged url into url array
readarray -t url < <(sqlite3 "${sqlite_file}" "SELECT ${BOOK_SLUG_RULE} FROM books ${SQLITE_QUERY}" | url_slug)
readarray -t directory < <(sqlite3 "${sqlite_file}" "SELECT directory FROM books ${SQLITE_QUERY}")
readarray -t pubdate < <(sqlite3 "${sqlite_file}" "SELECT pubdate FROM books ${SQLITE_QUERY}")
readarray -t lastmodified < <(sqlite3 "${sqlite_file}" "SELECT lastmodified FROM books ${SQLITE_QUERY}")
readarray -t lang < <(sqlite3 "${sqlite_file}" "SELECT lang FROM books ${SQLITE_QUERY}")
readarray -t subtitle < <(sqlite3 "${sqlite_file}" "SELECT subtitle FROM books ${SQLITE_QUERY}" | sed "s/'/\&#39;/g")
readarray -t SORTtopics < <(sqlite3 "${sqlite_file}" "SELECT SORTtopics FROM books ${SQLITE_QUERY}")
readarray -t publisher < <(sqlite3 "${sqlite_file}" "SELECT publisher FROM books ${SQLITE_QUERY}")
readarray -t publisher_url < <(sqlite3 "${sqlite_file}" "SELECT publisher FROM books ${SQLITE_QUERY}" | url_slug)
readarray -t notes < <(sqlite3 "${sqlite_file}" "SELECT notes FROM books ${SQLITE_QUERY}" | sed "s/'/\&#39;/g")
readarray -t isbn < <(sqlite3 "${sqlite_file}" "SELECT ISBN FROM books ${SQLITE_QUERY}")
readarray -t libgen_search < <(sqlite3 "${sqlite_file}" "SELECT libgen_search FROM books ${SQLITE_QUERY}")
readarray -t library < <(sqlite3 "${sqlite_file}" "SELECT library FROM books ${SQLITE_QUERY}")
readarray -t library_url < <(sqlite3 "${sqlite_file}" "SELECT library FROM books ${SQLITE_QUERY}" | url_slug)
readarray -t library_issue < <(sqlite3 "${sqlite_file}" "SELECT library_issue FROM books ${SQLITE_QUERY}")

# put notes into array as base64
#readarray -t notes < <(jq -r '.[].note | @base64' data.json)


# get book count info for loop
num_rows="${#title[@]}"
array_count="${_count:-$num_rows}"

if [[ -z "${_count}" ]]; then
    first_book=$((num_rows - array_count))
else
    first_book=$((num_rows - array_count - 1))
fi
last_book=$((num_rows - 1))

# get first entry after last entry
title[$((last_book + 1))]="${title[0]}"
author[$((last_book + 1))]="${author[0]}"
url[$((last_book + 1))]="${url[0]}"


# write book count info for external loop
echo "declare -A book
book[count]=\"${array_count}\"
book[first]=\"${first_book}\"
book[last]=\"${last_book}\"
"

# print array to stdout
for i in $(seq "${first_book}" "${last_book}"); do

    # seperate author if multiple
    if [[ "${author[$i]}" =~ "," ]]; then
        readarray -t comma_author <<<"${author[$i]//, /$'\n'}"
        readarray -t comma_author_url < <(echo "${author[$i]//, /$'\n'}" | url_slug )
        comma_author_count="${#comma_author[@]}"
        comma_author_last="$((comma_author_count - 1))"
echo "book[$i,author,count]=\"${comma_author_count}\"
book[$i,author,first]=\"0\"
book[$i,author,last]=\"${comma_author_last}\""
        for c in $(seq "0" "${comma_author_last}"); do
echo "book[$i,author,$c]=\"${comma_author[$c]}\"
book[$i,author,$c,url]=\"${comma_author_url[$c]}\""
        done
    else
echo "book[$i,author,count]=\"1\"
book[$i,author,first]=\"0\"
book[$i,author,last]=\"0\"
book[$i,author,0]=\"${author[$i]}\"
book[$i,author,0,url]=\"${author_url[$i]}\""
    fi


    # seperate publisher if multiple
    if [[ "${publisher[$i]}" =~ "," ]]; then
        readarray -t comma_publisher <<<"${publisher[$i]//, /$'\n'}"
        readarray -t comma_publisher_url < <(echo "${publisher[$i]//, /$'\n'}" | url_slug )
        comma_publisher_count="${#comma_publisher[@]}"
        comma_publisher_last="$((comma_publisher_count - 1))"
echo "book[$i,publisher,count]=\"${comma_publisher_count}\"
book[$i,publisher,first]=\"0\"
book[$i,publisher,last]=\"${comma_publisher_last}\""
        for c in $(seq "0" "${comma_publisher_last}"); do
echo "book[$i,publisher,$c]=\"${comma_publisher[$c]}\"
book[$i,publisher,$c,url]=\"${comma_publisher_url[$c]}\""
        done
    else
echo "book[$i,publisher,count]=\"1\"
book[$i,publisher,first]=\"0\"
book[$i,publisher,last]=\"0\"
book[$i,publisher,0]=\"${publisher[$i]}\"
book[$i,publisher,0,url]=\"${publisher_url[$i]}\""
    fi


    # seperate library if multiple
    if [[ -z "${library[$i]}" ]]; then
        library_count=0
    else
        library_count=1
    fi
    if [[ "${library[$i]}" =~ "," ]]; then
        readarray -t comma_library <<<"${library[$i]//, /$'\n'}"
        readarray -t comma_library_url < <(echo "${library[$i]//, /$'\n'}" | url_slug )
        comma_library_count="${#comma_library[@]}"
        comma_library_last="$((comma_library_count - 1))"
echo "book[$i,library,count]=\"${comma_library_count}\"
book[$i,library,first]=\"0\"
book[$i,library,last]=\"${comma_library_last}\""
        for c in $(seq "0" "${comma_library_last}"); do
echo "book[$i,library,$c]=\"${comma_library[$c]}\"
book[$i,library,$c,url]=\"${comma_library_url[$c]}\""
        done
    else
echo "book[$i,library,count]=\"${library_count}\"
book[$i,library,first]=\"0\"
book[$i,library,last]=\"0\"
book[$i,library,0]=\"${library[$i]}\"
book[$i,library,0,url]=\"${library_url[$i]}\""
    fi


echo "book[$i,title]=\"${title[$i]}\"
book[$i,subtitle]=\"${subtitle[$i]}\"
book[$i,author]=\"${author[$i]}\"
book[$i,SORTtopics]=\"${SORTtopics[$i]}\"
book[$i,publisher]=\"${publisher[$i]}\"
book[$i,lang]=\"${lang[$i]}\"
book[$i,uuid]=\"${uuid[$i]}\"
book[$i,pubdate]=\"${pubdate[$i]}\"
book[$i,lastmodified]=\"${lastmodified[$i]}\"
book[$i,directory]=\"${directory[$i]}\"
book[$i,notes]=\"${notes[$i]}\"
book[$i,isbn]=\"${isbn[$i]}\"
book[$i,libgen_search]=\"${libgen_search[$i]}\"
book[$i,library]=\"${library[$i]}\"
book[$i,library_issue]=\"${library_issue[$i]}\"
book[$i,row_id]=\"${i}\"
book[$i,url]=\"${url[$i]}\"
book[$i,prev,title]=\"${title[$((i - 1))]}\"
book[$i,prev,author]=\"${author[$((i - 1))]}\"
book[$i,prev,url]=\"${url[$((i - 1))]}\"
book[$i,next,title]=\"${title[$((i + 1))]}\"
book[$i,next,author]=\"${author[$((i + 1))]}\"
book[$i,next,url]=\"${url[$((i + 1))]}\"
"
done | sed 's/=\"null\"/=\"\"/g; s/=\"bnVsbA==\"/=\"\"/g'
